unit FormBg;

interface

uses Windows, NForm;

type
  TFormBg = class(TObject)
    Form: TForm;
    R, G, B, A: Integer;
    Width, Height: Integer;
    Bmp: HBITMAP;
    Bits: Pointer;
    destructor Destroy; override;
    procedure Make;
    procedure Paint(DC: HDC);
  end;

implementation

procedure AlphaBlend(Bits: Pointer; W, H, R, G, B, Alpha: Integer);
type
  PTriByte = ^TTriByte;
  TTriByte = array[0..3] of Byte;
var
  x, y, pad: Integer;
  p: PByte;
begin
  pad := (W * 3) mod 4;
  p := Bits;
  for y := 0 to H-1 do
  begin
    for x := 0 to W-1 do
    begin
      PTriByte(p)[0] := B + (PTriByte(p)[0] - B) * Alpha shr 8;
      PTriByte(p)[1] := G + (PTriByte(p)[1] - G) * Alpha shr 8;
      PTriByte(p)[2] := R + (PTriByte(p)[2] - R) * Alpha shr 8;
      Inc(p, 3);
    end;
    Inc(p, pad);
  end;
end;

destructor TFormBg.Destroy;
begin
  if Bmp <> 0 then
    DeleteObject(Bmp);
end;

procedure TFormBg.Make;
var
  info: TBitmapInfo;
  dc, mdc: HDC;
  x, y, w, h: Integer;
  oldbmp: HBITMAP;
begin
  dc := GetDC(Form.Handle);
  Form.GetClientSize(w, h);
  if (Bmp <> 0) and (Width <> w) or (Height <> h) then
  begin
    DeleteObject(Bmp);
    Bmp := 0;
  end;
  if Bmp = 0 then
    with info.bmiHeader do
    begin
      biSize := SizeOf(info.bmiHeader);
      biWidth := w;
      biHeight := h;
      biPlanes := 1;
      biBitCount := 24;
      biCompression := BI_RGB;
      Bmp := CreateDIBSection(dc, info, DIB_RGB_COLORS, Bits, 0, 0);
    end;
  Form.ClientToScreen(0, 0, x, y);
  SetBrushOrgEx(dc, -(x div 8), -(y div 8), nil);
  mdc := CreateCompatibleDC(dc);
  oldbmp := SelectObject(mdc, Bmp);
  PaintDesktop(dc);
  BitBlt(mdc, 0, 0, w, h, dc, 0, 0, SRCCOPY);
  SelectObject(mdc, oldbmp);
  DeleteDC(mdc);
  ReleaseDC(Form.Handle, dc);
  AlphaBlend(Bits, w, h, R, G, B, A);
end;

procedure TFormBg.Paint(DC: HDC);
var
  mdc: HDC;
  oldbmp: HBITMAP;
  w, h: Integer;
begin
  if Bmp = 0 then
    Make;
  mdc := CreateCompatibleDC(DC);
  oldbmp := SelectObject(mdc, Bmp);
  Form.GetClientSize(w, h);
  BitBlt(DC, 0, 0, w, h, mdc, 0, 0, SRCCOPY);
  SelectObject(mdc, oldbmp);
  DeleteDC(mdc);
end;

end.
